/*******************************************************
 *   
 *
 * Copyright (C) 2002 OSS Nokalva, Inc.  All rights reserved
 *
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS Nokalva, INC. AND
 * MAY BE USED ONLY BY DIRECT LICENSEES OF OSS Nokalva, INC.
 * THIS FILE MAY NOT BE DISTRIBUTED. 
 *
 * Prepared by OSS Nokalva, Inc.  1/3/2002  
 *
 *******************************************************

  OSS ASN.1/Pure Java Example For The Protocols ROS+H4501+H323+H235+H225
 
  Description
 
  This example is for use with the OSS ASN.1 Pure Java Tools. 
  It illustrates how you can automatically encode and decode 
  the H4501SupplementaryService PDU and print all its components.
 
  The original file H4501-Supplementary-ServiceAPDU-Structure.asn was
  missing the OPERATIONS information object set. It was empty. So the
  OPERATIONS information object was added to make it possible to
  automatically encode/decode the inner sub-PDU carried by OpenType.
  Note that the following ASN.1 Pure Java compiler command line was used:
 
  asn1pjav *.asn -root -per -paramtypesuperclass -out h450
 *
 ************************************************************/


import java.io.*;

import com.oss.asn1.*;
import com.oss.util.HexTool;

import h450.*;
import h450.h4501_supplementary_serviceapdu_structure.*;
import h450.remote_operations_apdus.*;
import h450.remote_operations_information_objects.*;

import h450.h323_messages.*;
import h450.addressing_data_elements.*;
import h450.call_transfer_operations.*;

public class Th450 {

    public static void main(String[] args) {

	// Initialize the project
	try {
	    H450.initialize();
	} catch (Exception e) {
	    System.out.println("Initialization exception: " + e);
	    System.exit(1);
	}

	Coder coder = H450.getPERAlignedCoder();

	coder.enableEncoderDebugging();
	coder.enableDecoderDebugging();
	coder.enableAutomaticEncoding();
	coder.enableAutomaticDecoding();

	H4501SupplementaryService ss =
	    H4501_Supplementary_ServiceAPDU_Structure.ss;

	test(coder, ss);
	test1(coder);
    }

    public static void test(Coder coder, H4501SupplementaryService ss)
    {
	// print the PDU
	System.out.println("\tUnencoded PDU...\n");
	System.out.println(ss);

	// encode the PDU
	System.out.println("\n\tEncoding H4501SupplementaryService With All the Nested Open Types...\n");
	byte[] encoding = null;
	try {
	    ByteArrayOutputStream sink = new ByteArrayOutputStream();

	    coder.encode(ss, sink);

	    // Extract the encoding from the sink stream
	    encoding = sink.toByteArray();

	} catch (Exception e) {
	    System.out.println("Encoding failed: " + e);
	    System.exit(1);
	}

	System.out.println("\nDecoding H4501SupplementaryService With All the Nested Open Types...\n");
	H4501SupplementaryService pdu = null;
	try {
	    ByteArrayInputStream source = new ByteArrayInputStream(encoding);

	    pdu = (H4501SupplementaryService)coder.decode(source, ss);

	} catch (Exception e) {
	    System.out.println("Decoding failed: " + e);
	    System.exit(1);
	}

	// print the PDU
	System.out.println("\tUnencoded PDU...\n");
	System.out.println(pdu);

	System.out.println("\n\tComponent ServiceApdus...\n");
	ServiceApdus sapdus = pdu.getServiceApdu();
	System.out.println(sapdus);

	System.out.println("\n\tCHOICE Selection rosApdus (SEQ OF ROS)...\n");
	ServiceApdus.RosApdus rapdus = (ServiceApdus.RosApdus)sapdus.getChosenValue();
	System.out.println(rapdus);

	System.out.println("\n\tComponent #0 Of SEQUENCE OF ROS...\n");
	ROS zero = rapdus.get(0);
	System.out.println(zero);

	System.out.println("\n\tCHOICE Selection Invoke...\n");
	Invoke invoke = (Invoke)zero.getChosenValue();
	System.out.println(invoke);

	System.out.println("\n\tComponent opcode (CHOICE) of Invoke...\n");
	Code oc = invoke.getOpcode();
	System.out.println(oc);

	System.out.println("\n\tSelection global (OID) of Code (CHOICE)...\n");
	ObjectIdentifier oid = (ObjectIdentifier)oc.getChosenValue();
	System.out.println(oid);

	System.out.println("\n\tOpen Type Component argument of Invoke...\n");
	OpenType arg = invoke.getArgument();
	System.out.println(arg);

	System.out.println("\n\tActual Type Password-Name Of Open Type Component...\n");
	Password_Name pn = (Password_Name)arg.getDecodedValue();
	System.out.println(pn);

	System.out.println("\n\tGet the password Component Reference...\n");
	OctetString ps = pn.getPassword();
	System.out.println(ps);

	System.out.println("\n\tSet the password Component to a New Value...\n");
	byte[] val = new byte [] {(byte)0xFF};
	ps.setValue(val);
	System.out.println(ps);

	System.out.println("\n\tSet the opcode Component to a New Value...\n");
	try {
	    oc.setGlobal(new ObjectIdentifier("{ 1 1 1}"));
	} catch (BadObjectIdentifierException e) {
	    System.out.println(e);
	}

	System.out.println("\n\tChanged PDU...\n");
	pdu.setServiceApdu(sapdus);
	System.out.println(pdu);
    }

    public static void test1(Coder coder)
    {
	System.out.println("\n\tConstruct H4501SupplementaryService PDU for Encoding...\n");
	AliasAddress e164address = AliasAddress.createAliasAddressWithE164(
	   new IA5String("1234567890"));

	BMPString abc = new BMPString("chewk");
	//set H323ID
	AliasAddress h323id = AliasAddress.createAliasAddressWithH323_ID(abc);;

	//set Endpoint address
	EndpointAddress.DestinationAddress ep = new EndpointAddress.DestinationAddress();
	ep.add(e164address);
	ep.add(h323id);

	EndpointAddress reroutingNo = new EndpointAddress(ep);

	CTInitiateArg ctarg = new CTInitiateArg();

	//set callIdentity
	ctarg.setCallIdentity(new CallIdentity("1234"));
	ctarg.setReroutingNumber(reroutingNo);

	Code cd1 = Code.createCodeWithLocal(9);

	Invoke inv = new Invoke();;
	inv.setArgument(new OpenType(ctarg));
	inv.setInvokeId(new InvokeId(1));
	inv.setOpcode(cd1);
		
	ROS ros = ROS.createROSWithInvoke(inv);
	
	ServiceApdus.RosApdus rapdus = new ServiceApdus.RosApdus();
	rapdus.add(ros);

	ServiceApdus sapdus = ServiceApdus.createServiceApdusWithRosApdus(rapdus);

	H4501SupplementaryService ss1 = new H4501SupplementaryService(sapdus);

	System.out.println(ss1);

	byte[] encoding = null;
	try {
	    ByteArrayOutputStream sink = new ByteArrayOutputStream();

	    coder.encode(ss1, sink);

	    // Extract the encoding from the sink stream
	    encoding = sink.toByteArray();

	} catch (Exception e) {
	    System.out.println("Encoding failed: " + e);
	    System.exit(1);
	}
	HexTool.printHex(encoding);
    }
}
